import re
from abc import ABCMeta
from brief import Brief, Fetch

class PythonPackgramBrief(Brief, metaclass=ABCMeta):
    
    #Public
  
    author = Fetch(
        file_pattern='AUTHORS.rst', 
        text_pattern=r'\n-\s+(.*?)\s+<.*?@.*?>',
        processors=[
            lambda items: items[0],
        ],
    ) 
         
    author_email = Fetch(
        file_pattern='AUTHORS.rst',
        text_pattern=r'\n-\s+.*?\s+<(.*?@.*?)>',    
        processors=[
            lambda items: items[0],
        ],
    )   
    
    classifiers = Fetch(
        file_pattern='README.rst',
        text_pattern=r'- ([^\n]* :: [^\n]*)',       
    )
    
    description = Fetch(
        file_pattern='README.rst',
        text_pattern=r'.*?=\s+(.*?)(?:\n|$)',    
        processors=[
            lambda items: items[0],
        ],
    )
    
    license = Fetch(
        file_pattern='LICENSE.rst',
        text_pattern=r'(.*?)\s+=',
        processors=[
            lambda items: items[0],
        ],
    )
    
    long_description = Fetch(
        file_pattern='README.rst',
        text_pattern=r'(.*)',
        text_pattern_flags=re.DOTALL,   
        processors=[
            lambda items: items[0],
        ],
    )    
     
    maintainer = Fetch(
        file_pattern='AUTHORS.rst',
        text_pattern=r'\n-\s+(.*?)\s+<.*?@.*?>',  
        processors=[
            lambda items: items[0],
        ],
    ) 
       
    maintainer_email = Fetch(
        file_pattern='AUTHORS.rst',
        text_pattern=r'\n-\s+.*?\s+<(.*?@.*?)>',   
        processors=[
            lambda items: items[0],
        ],
    )
       
    name = Fetch(
        file_pattern='README.rst',
        text_pattern=r'(.*?)\s+=',
        processors=[
            lambda items: items[0],
            str.lower,
        ],
    )
    
    version = Fetch(
        file_pattern='.*version.py',
        text_pattern=r'(?:major|minor|micro|level)\s*=\s*[\'"]?(\w*)[\'"]?',
        processors=[
            lambda items: [item for item in items if item != 'final'],
            lambda items: '.'.join(items),
        ],
    )                        