import sys
from run import Run
from pprint import pprint
from subprocess import call
from abc import ABCMeta, abstractmethod

class PythonPackgramRun(Run, metaclass=ABCMeta):
    
    #Public

    @property
    @abstractmethod
    def package(self):
        pass #pragma: no cover
    
    def name(self):
        "Print package name"
        print(self.package.get('name', None))
    
    def version(self):
        "Print package version"
        print(self.package.get('version', None))
    
    def summary(self):
        "Print package summary"
        pprint(self.package)
                  
    def test(self):
        "Test package"
        #TODO: check tests existing
        return call(['nosetests-3.3-dev', 'tests', '-sv'])
     
    def build(self):
        "Build package"
        call(['python3.3', 'setup.py', 'sdist'])
        
    def register(self):
        "Upload package"
        call(['python3.3', 'setup.py', 'sdist', 'register', 'upload'])
          
    def clean(self):
        "Clean from dist, build, eggs"
        command = ' '.join(['rm', '-rfv', 'dist', 'build', '*.egg-info'])
        call(command, shell=True)
    
    def install(self):
        "Install package"
        self.build()
        command = ' '.join(['sudo', 'pip3', 'install', 'dist/*'])
        call(command, shell=True)

    def uninstall(self):
        "Uninstall package"
        call(['sudo', 'pip3', 'uninstall', '-y', self.package['name']])  
        
#     def release(self, step, level='final', register=False):
#         """
#         Release package with version step and level.
#         Steps:
#         - major
#         - minor
#         - micro
#         - level
#         """
#         release = self.package.release(step, level=level)
#         if release.version1 == release.version2:
#             sys.exit('Incorrect step/level')  
#         if self.test() != 0:
#             sys.exit('Tests failed')    
#         if not self._confirm(
#             'Release version {version}?'.
#             format(version=release.version2)
#         ):
#             sys.exit('Aborted by user')
#         self._commit('prepared to release')
#         release.apply()
#         if self.test() != 0:
#             sys.exit('Tests failed')         
#         self._diff()
#         if not self._confirm((
#             'Version has been updated to {version}\n'
#             'See code difference above\n'
#             'Continue?'
#         ).format(
#             version=self.package['version'])
#         ):
#             self.checkout('.')
#             sys.exit(
#                 'Rolling back\n'
#                 'Aborted by user'
#             )
#         self._commit('updated version')
#         self._push('develop')
#         self._checkout('master')
#         self._merge('develop')
#         self._tag(self.package['version'])
#         self._push('master')
#         self._checkout('develop')
#         if register:        
#             self.register()
            
    #Protected
    
    def _diff(self):
        call(['git', 'diff'])
                   
    def _commit(self, message):
        call(['git', 'commit', '-am', message])
        
    def _tag(self, name):
        call(['git', 'tag', name])
        
    def _checkout(self, pointer):
        call(['git', 'checkout', pointer])
        
    def _merge(self, branch):
        call(['git', 'merge', branch])
        
    def _push(self, branch):
        call(['git', 'push', 'origin', branch, '--tags'])
    
    def _confirm(self, text):
        response = input('{text} [y/n]: '.format(text=text))
        if response.lower() == 'y':
            return True
        else:
            return False              